"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.isOfficialComponentLib = exports.readComponentLibMata = exports.writeLibCommonRes2file = exports.getYyptConfigInfo = exports.getScopeComponents = exports.getComponentsInfo = exports.getInputProps = exports.removeRequireUncached = exports.requireUncached = exports.JsonToStringWithVariableName = exports.getSelfPackageJson = exports.getCurrentPackageJson = exports.promiseWrapper = exports.getFileNameByUrl = exports.deepDealSchema = exports.simpleDeepClone = exports.deepDeal = exports.isPlainObject = exports.isArray = exports.getMetaInfoBySourceKey = void 0;
const path_1 = __importDefault(require("path"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const style_1 = require("./style");
const generateFiles_1 = require("./generateFiles");
const cals_1 = require("@cloudbase/cals");
var common_1 = require("../../generator/util/common");
Object.defineProperty(exports, "getMetaInfoBySourceKey", { enumerable: true, get: function () { return common_1.getMetaInfoBySourceKey; } });
Object.defineProperty(exports, "isArray", { enumerable: true, get: function () { return common_1.isArray; } });
Object.defineProperty(exports, "isPlainObject", { enumerable: true, get: function () { return common_1.isPlainObject; } });
Object.defineProperty(exports, "deepDeal", { enumerable: true, get: function () { return common_1.deepDeal; } });
Object.defineProperty(exports, "simpleDeepClone", { enumerable: true, get: function () { return common_1.simpleDeepClone; } });
Object.defineProperty(exports, "deepDealSchema", { enumerable: true, get: function () { return common_1.deepDealSchema; } });
Object.defineProperty(exports, "getFileNameByUrl", { enumerable: true, get: function () { return common_1.getFileNameByUrl; } });
const os_1 = __importDefault(require("os"));
const homeDir = os_1.default.homedir();
const commandConfigPath = path_1.default.join(homeDir, '.warc');
function promiseWrapper(p) {
    return new Promise((resolve) => {
        try {
            p.then((i) => resolve([null, i])).catch((e) => resolve([e, null]));
        }
        catch (e) {
            resolve([e, null]);
        }
    });
}
exports.promiseWrapper = promiseWrapper;
function getCurrentPackageJson() {
    try {
        const { name, version } = fs_extra_1.default.readJSONSync(path_1.default.resolve(process.cwd(), 'package.json'));
        return {
            name,
            version,
        };
    }
    catch (e) { }
}
exports.getCurrentPackageJson = getCurrentPackageJson;
function getSelfPackageJson() {
    try {
        return fs_extra_1.default.readJSONSync(path_1.default.resolve(__dirname, '../../package.json'));
    }
    catch (e) { }
}
exports.getSelfPackageJson = getSelfPackageJson;
function JsonToStringWithVariableName(copyJson) {
    return JSON.stringify(copyJson, null, 2).replace(/"%%%(.*?)%%%"/g, function (match, expression) {
        return expression
            .replace(/\\"/g, '"')
            .replace(/\\'/g, "'")
            .replace(/\\r/g, '\r')
            .replace(/\\n/g, '\n');
    });
}
exports.JsonToStringWithVariableName = JsonToStringWithVariableName;
function requireUncached(module) {
    delete require.cache[require.resolve(module)];
    return require(module);
}
exports.requireUncached = requireUncached;
function removeRequireUncached(path = '') {
    if (fs_extra_1.default.existsSync(path)) {
        delete require.cache[require.resolve(path)];
    }
}
exports.removeRequireUncached = removeRequireUncached;
function getInputProps(componentsMetaMap) {
    var _a, _b;
    const outputObj = {};
    for (let key in componentsMetaMap) {
        let component = componentsMetaMap[key];
        const sourceKey = key;
        if (component.isComposite) {
            let compItem = component;
            Object.keys(compItem.dataForm || {}).forEach((key) => {
                var _a, _b;
                const inputProps = ((_a = compItem.dataForm[key]) === null || _a === void 0 ? void 0 : _a.inputProp) ||
                    ((_b = compItem.dataForm[key]) === null || _b === void 0 ? void 0 : _b.syncProps);
                if (inputProps) {
                    outputObj[sourceKey] = Object.assign({ [key]: inputProps }, (outputObj[sourceKey] || {}));
                }
            });
        }
        else {
            const inputProps = ((_a = component === null || component === void 0 ? void 0 : component.meta) === null || _a === void 0 ? void 0 : _a.inputProps) || ((_b = component === null || component === void 0 ? void 0 : component.meta) === null || _b === void 0 ? void 0 : _b.syncProps);
            if (inputProps) {
                outputObj[sourceKey] = inputProps;
            }
        }
    }
    return outputObj;
}
exports.getInputProps = getInputProps;
function getComponentsInfo(appBuildDir, dependencies) {
    return __awaiter(this, void 0, void 0, function* () {
        const outputObj = {};
        yield Promise.all(dependencies.map(({ name: materialName, version, components, isComposite }) => __awaiter(this, void 0, void 0, function* () {
            if (isComposite) {
                components.forEach((component) => {
                    let compItem = component;
                    const sourceKey = `${materialName}:${compItem.name}`;
                    outputObj[sourceKey] = Object.assign({ isComposite }, compItem);
                });
            }
            else {
                const materialComponentsPath = path_1.default
                    .resolve(appBuildDir, `libraries/${materialName}@${version}/components`)
                    .replace(/packages\/\w+\//, '');
                const componentLibPath = path_1.default
                    .resolve(appBuildDir, `libraries/${materialName}@${version}`)
                    .replace(/packages\/\w+\//, '');
                const meta = readComponentLibMata(componentLibPath);
                if (meta) {
                    const { components: componentsMap } = meta;
                    for (let name in componentsMap) {
                        const sourceKey = `${materialName}:${name}`;
                        let metaJson = componentsMap[name];
                        outputObj[sourceKey] = Object.assign({ isComposite }, metaJson);
                    }
                }
                else {
                    const components = yield fs_extra_1.default.readdir(materialComponentsPath);
                    yield Promise.all(components.map((name) => __awaiter(this, void 0, void 0, function* () {
                        const sourceKey = `${materialName}:${name}`;
                        const componentMetaPath = `${materialComponentsPath}/${name}/meta.json`;
                        let metaJson = {
                            name,
                            meta: yield fs_extra_1.default.readJson(componentMetaPath),
                        };
                        outputObj[sourceKey] = Object.assign(Object.assign({ isComposite }, metaJson), { compConfig: {} });
                    })));
                }
            }
        })));
        return outputObj;
    });
}
exports.getComponentsInfo = getComponentsInfo;
function getScopeComponents(appBuildDir, dependencies) {
    return __awaiter(this, void 0, void 0, function* () {
        const componentsScope = {};
        try {
            const components = yield getComponentsInfo(appBuildDir, dependencies);
            Object.entries(components).map(([key, value]) => {
                var _a;
                const dataForm = (value === null || value === void 0 ? void 0 : value.dataForm) || ((_a = value === null || value === void 0 ? void 0 : value.data) === null || _a === void 0 ? void 0 : _a.properties);
                if (typeof dataForm === 'object') {
                    const dataFormOmit = Object.keys(dataForm).reduce((acc, prop) => {
                        var _a;
                        ((_a = dataForm[prop]) === null || _a === void 0 ? void 0 : _a['x-slot-scope']) && (acc[prop] = dataForm[prop]);
                        return acc;
                    }, {});
                    Object.keys(dataFormOmit).length &&
                        (componentsScope[key] = { dataForm: dataFormOmit });
                }
            });
        }
        catch (e) { }
        return componentsScope;
    });
}
exports.getScopeComponents = getScopeComponents;
function getYyptConfigInfo(extraData) {
    return __awaiter(this, void 0, void 0, function* () {
        let configJson;
        try {
            configJson = yield fs_extra_1.default.readJSON(commandConfigPath);
        }
        catch (e) { }
        configJson = configJson || {
            yyptAppKey: '',
            reportUrl: '',
            stopReport: false,
        };
        if (!extraData || !extraData.operationService) {
            extraData = extraData || {};
            extraData.operationService = extraData.operationService || {};
        }
        const yyptAppKey = configJson.yyptAppKey || extraData.operationService.extAppId || '';
        const reportUrl = configJson.reportUrl || extraData.operationService.reportUrl || '';
        const stopReport = configJson.stopReport === 'true' || !extraData.operationService.yyptEnabled;
        return {
            yyptAppKey,
            reportUrl,
            stopReport,
        };
    });
}
exports.getYyptConfigInfo = getYyptConfigInfo;
function writeLibCommonRes2file(gItem, codeDir) {
    return __awaiter(this, void 0, void 0, function* () {
        const compLibCommonResource = gItem.compLibCommonResource;
        const libCommonResFiles = [];
        libCommonResFiles.push({
            path: path_1.default.join(codeDir, `class.${codeDir.includes('/mp/') ? 'wxss' : 'less'}`),
            code: compLibCommonResource
                ? yield (0, style_1.processLess)(`
        ${compLibCommonResource.theme.variable || ''}
        ${compLibCommonResource.class || ''}
        ${compLibCommonResource.theme.class || ''}
        `)
                : '',
        }, {
            path: path_1.default.join(codeDir, 'const.js'),
            code: compLibCommonResource ? compLibCommonResource.const.code : '',
        }, {
            path: path_1.default.join(codeDir, 'tools.js'),
            code: compLibCommonResource ? compLibCommonResource.tools.code : '',
        });
        yield Promise.all(libCommonResFiles.map((item) => (0, generateFiles_1.writeFile)(item.path, item.code)));
    });
}
exports.writeLibCommonRes2file = writeLibCommonRes2file;
function readComponentLibMata(libDir) {
    var _a;
    let metaPath = path_1.default.join(libDir, 'meta.json');
    let isExistsMeta = fs_extra_1.default.existsSync(metaPath);
    if (!isExistsMeta) {
        return null;
    }
    let meta = fs_extra_1.default.readJSONSync(metaPath);
    let [major] = ((_a = meta === null || meta === void 0 ? void 0 : meta.schemaVersion) === null || _a === void 0 ? void 0 : _a.split('.')) || [];
    const originComponentMetaMap = meta.components;
    if (Number(major) >= 2) {
        meta = (0, cals_1.deserializeComponentLibraryMeta)(meta);
    }
    for (let key in meta.components) {
        meta.components[key] = Object.assign(Object.assign({}, originComponentMetaMap[key]), { name: key, meta: meta.components[key] });
    }
    return meta;
}
exports.readComponentLibMata = readComponentLibMata;
const _OFFICIAL_COMPONENT_LIB = {
    'gsd-h5-react': ['0.0.61', '0.0.70'],
    CLOUDBASE_BUSSINESS: ['1627377179261'],
};
function isOfficialComponentLib(name, version) {
    var _a, _b;
    return (!!((_a = _OFFICIAL_COMPONENT_LIB[name]) === null || _a === void 0 ? void 0 : _a.includes(version)) ||
        ((_b = _OFFICIAL_COMPONENT_LIB[name]) === null || _b === void 0 ? void 0 : _b.includes('*')));
}
exports.isOfficialComponentLib = isOfficialComponentLib;
