import { CSSProperties, ICommonStyle } from './style';
import { ActionType } from './action';
import { IWeAppCode } from './lowcode';
import { IMiniprogramPlugin, IPlugin } from '../types/plugins';
import { HISTORY_TYPE } from '../../types';
export interface IAppAndPageVar {
    dataset?: {
        state?: {};
        param?: {};
    };
    vars: {
        data: any[];
    };
}
export interface IWeAppData {
    mpPkgUrl?: string;
    selectedPageId?: string;
    historyType?: HISTORY_TYPE;
    pageInstanceList: IWeAppPage[];
    lowCodes: IWeAppCode[];
    npmDependencies: {
        [packageName: string]: string;
    };
    plugins: IPlugin[];
    maxID?: number;
    rootPath?: string;
    themeVars?: {
        [key: string]: string;
    };
    presetColors?: string[];
    appConfig?: Record<string, any>;
    envId: string;
    datasources: any[];
    dataset?: IAppAndPageVar['dataset'];
    vars: IAppAndPageVar['vars'];
    miniprogramPlugins: IMiniprogramPlugin[];
    extra: {
        domain: string;
        clientId?: string;
        loginConfigVersion?: string;
    };
}
export interface IWeAppPage {
    id: string;
    isHome: boolean;
    data: {
        [prop: string]: IDynamicValue;
    };
    listeners: IEventListener[];
    componentInstances: {
        [key: string]: IWeAppComponentInstance;
    };
    pluginInstances: any[];
    commonStyle: ICommonStyle;
    styleBindPath: string;
    lowCodes?: IWeAppCode[];
    children?: IWeAppPage[];
    pageId?: string;
    codeType?: 'page' | 'global';
    code?: string;
    dataset?: IAppAndPageVar['dataset'];
}
export interface IWeAppComponentInstance {
    xComponent?: {
        moduleName: string;
        name: string;
    };
    xProps?: {
        data: {
            [prop: string]: IDynamicValue;
        };
        directives?: {
            waIf?: IDynamicValue;
            waFor?: IDynamicValue;
        };
        listeners?: IEventListener[];
        style: CSSProperties;
        styleBind: IDynamicValue;
        classList: string[];
        classListBind: IDynamicValue;
        commonStyle: ICommonStyle;
        styleBindPath?: string;
        customDataForm: Record<string, any>;
        staticResourceAttribute: string[];
    };
    xIndex?: number;
    properties?: {
        [key: string]: IWeAppComponentInstance;
    };
    genericComp?: {
        propName: string;
        title: string;
        description?: string;
        allowedCompType?: string;
        xIndex?: number;
    };
}
export interface IEventListener extends IEventModifiers {
    key?: string;
    trigger: string;
    type: ActionType;
    handler?: {
        moduleName: string;
        name: string;
    };
    jsCode?: string;
    data: {
        [prop: string]: IDynamicValue;
    };
}
export interface IEventModifiers {
    isCapturePhase?: boolean;
    noPropagation?: boolean;
}
export interface IDynamicValue {
    type?: 'static' | PropBindType;
    value: any | string;
}
export declare enum PropBindType {
    state = "rematch",
    computed = "computed",
    forItem = "for-item",
    expression = "expression",
    prop = "prop",
    slot = "slot",
    dataVar = "dataVar",
    stateData = "state",
    paramData = "params",
    scope = "scope"
}
export declare enum BindPropertyPath {
    style = "style",
    classNameList = "classNameList"
}
export interface ILockedPage {
    id: string;
    username: string;
    socketId: string;
}
export declare type IBuildType = 'web' | 'mp' | 'app' | 'qywxH5' | 'wxH5';
//# sourceMappingURL=app.d.ts.map