"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.checkVisible = exports.isValidClassNameListBind = exports.isValidStyleBind = exports.setValidValue = exports.isEmptyObj = exports.isPlainObject = exports.isArray = exports.camelcase = exports.kebabCase = exports.loopDealWithFn = void 0;
const config_1 = require("../config");
const types_1 = require("../types");
function loopDealWithFn(data, runFn, result = []) {
    data.map(item => {
        if (item.children) {
            loopDealWithFn(item.children, runFn, result);
        }
        result.push(runFn(item));
    });
    return result;
}
exports.loopDealWithFn = loopDealWithFn;
function kebabCase(str) {
    return str.replace(config_1.KEBAB_REGEX, function (match) {
        return '-' + match.toLowerCase();
    });
}
exports.kebabCase = kebabCase;
function camelcase(str, firstUpperCase = false) {
    str = str.replace(/[_-]([a-z])/g, function (l) {
        return l[1].toUpperCase();
    });
    if (firstUpperCase)
        str = str.charAt(0).toUpperCase() + str.slice(1);
    return str;
}
exports.camelcase = camelcase;
function isArray(src) {
    return Object.prototype.toString.call(src) === '[object Array]';
}
exports.isArray = isArray;
function isPlainObject(src) {
    return Object.prototype.toString.call(src) === '[object Object]';
}
exports.isPlainObject = isPlainObject;
const isEmptyObj = (obj) => {
    if (!isPlainObject(obj)) {
        return false;
    }
    for (const i in obj) {
        if (Object.prototype.hasOwnProperty.call(obj, i)) {
            return false;
        }
    }
    return true;
};
exports.isEmptyObj = isEmptyObj;
function setValidValue(target, key, value) {
    if (value === undefined || value === null) {
        return;
    }
    if (isArray(value) && value.length === 0) {
        return;
    }
    if (isPlainObject(value) && (0, exports.isEmptyObj)(value)) {
        return;
    }
    if (!target) {
        return;
    }
    target[key] = value;
}
exports.setValidValue = setValidValue;
function isValidStyleBind(styleBind) {
    if (!styleBind) {
        return false;
    }
    if (styleBind.propertyPath === types_1.BindPropertyPath.style && styleBind.bindDataPath) {
        return true;
    }
    return false;
}
exports.isValidStyleBind = isValidStyleBind;
function isValidClassNameListBind(classNameListBind) {
    if (!classNameListBind) {
        return false;
    }
    if (classNameListBind.propertyPath === types_1.BindPropertyPath.classNameList) {
        return true;
    }
    return false;
}
exports.isValidClassNameListBind = isValidClassNameListBind;
function checkVisible({ _visible }) {
    return _visible !== false && _visible !== '';
}
exports.checkVisible = checkVisible;
